Class: Channel::Mattermost::V3
- Inherits:
-
Slack::V3
- Object
- Slack::V3
- Channel::Mattermost::V3
- Defined in:
- app/models/pager_tree/integrations/channel/mattermost/v3.rb
Instance Method Summary collapse
-
#_alert_created_at_timestamp_formatted ⇒ Object
we can just piggy back the Channel::Slack::V3 since Mattermost is a clone of Slack both integrations have the same exact functionality.
Instance Method Details
#_alert_created_at_timestamp_formatted ⇒ Object
we can just piggy back the Channel::Slack::V3 since Mattermost is a clone of Slack both integrations have the same exact functionality
6 7 8 |
# File 'app/models/pager_tree/integrations/channel/mattermost/v3.rb', line 6 def _alert.created_at.utc.iso8601.to_s end |